#include "aap_ana.h"
#include "cfg.h"
#include "arec.h"
#include "ana_req.h"
#include <stdlib.h>

/* FIXME: Just doing this since aapd had it done. In reality there may be
   alignment issues causing the members of the struct to have space between
   them. */
struct ana_raw {
    unsigned int current_time;
    unsigned int address_count;
    unsigned int end_time;
};

static int send_msg(aap_ana *a){
    struct ana_raw ana;
    aap_msg *msg;
    int len = 12;
    
    /* insert current time */
    ana.current_time = htonl(time(NULL));
    ana.address_count = htonl(a->ac);
    ana.end_time = htonl(a->et);

    msg = aap_msg_new();
    msg->create(msg, AAP_MSG_ANA, AAP_MSG_IP,
		a->rseq, a->mseq, (char *)&ana, len);
    msg->send(msg, a->n, a->ns);
    a->mseq++;
    
    return 0;
}

static int ana_resend_callback(void *obj, unsigned int time){
    /* First send the message again. */
    aap_ana *ana_msg = (aap_ana *)obj;
    ana_msg->send(ana_msg);
    /* Increase the wait until the next retransmission. */
    ana_msg->resend_time = ana_msg->resend_time * 2;
    if(ana_msg->resend_time < ana_msg->c->ANNOUNCE_WAIT){
        ana_msg->t->set(ana_msg->t, ana_msg->resend_time,
                        ana_msg->resender, ana_msg);
    } else {
        ana_msg->delete(ana_msg);
    }
    
    return 0;
}

static void delete(aap_ana *a){
    free(a);
}

int aap_ana_receive(aap *aap, aap_msg *msg) {
    struct ana_raw *ana;
    int len;
    unsigned int clock_skew;
    ana_req *ana_r;

    msg->body(msg, (char **)&ana, &len);
    if(len != 12){
        return 1;
        DEBUG('d', "aap_ana: Received malformed message.\n");
    }
    clock_skew = ntohl(ana->current_time) - time(NULL);

    /* Now do stuff in accordance with the ANA. */
    ana_r = ana_req_create(ntohl(ana->address_count),
                           ntohl(ana->end_time),
                           clock_skew,
                           msg->rseq(msg));
    aap->arec->insert_ana(aap->arec, ana_r);
    return 0;
}

aap_ana *aap_ana_new(net *n, net_socket *ns, tim *t, cfg *c, aap *aap,
                     unsigned int ac, unsigned int et, int rseq){
    aap_ana *ana = malloc(sizeof(aap_ana));
    
    ana->n = n;
    ana->ns = ns;
    ana->t = t;
    ana->c = c;
    ana->aap = aap;
    ana->ac = ac;
    ana->et = et;
    ana->rseq = rseq;
    ana->mseq = 0;
    ana->resend_time = c->RESEND_WAIT;
    
    ana->send = send_msg;
    ana->delete = delete;
    ana->resender = ana_resend_callback;
    
    return ana;
}
