#include "aap_asrp.h"
#include "list.h"
#include <stdlib.h>

/* FIXME: Just doing this since aapd had it done. In reality there may be
   alignment issues causing the members of the struct to have space between
   them. */
typedef struct asrp_report{
    unsigned int first;
    unsigned int last;
    unsigned int in_use_count;
} asrp_report;

typedef struct asrp_request{
    unsigned int addrs_requested;
    unsigned int end_time_requested;
} asrp_request;

static char *add_raw8(char data, char *raw_message, int *index);

static char *add_raw32(long data, char *raw_message, int *index);

static int send_msg(aap_asrp *a){
    aap_msg *msg;
    int i = 0;
    int len = 4 + 1 + a->reports->length(a->reports) * 12 +
        a->requests->length(a->requests) * 8;
    char *asrp = NULL;
    int index = 0;
    asrp_request *request;
    asrp_report *report;
    
    asrp = add_raw32(htonl(time(NULL)), asrp, &index);
    asrp = add_raw8(a->reports->length(a->reports), asrp, &index);
    for(i = 0;i < a->reports->length(a->reports);i++){
        report = (asrp_report *)a->reports->elem_at(a->reports, i);
        asrp = add_raw32(report->first, asrp, &index);
        asrp = add_raw32(report->last, asrp, &index);
        asrp = add_raw32(report->in_use_count, asrp, &index);
    }
    asrp = add_raw8(a->requests->length(a->requests), asrp, &index);
    for(i = 0;i < a->requests->length(a->requests);i++){
        request = (asrp_request *)a->requests->elem_at(a->requests, i);
        asrp = add_raw32(request->addrs_requested, asrp, &index);
        asrp = add_raw32(request->end_time_requested, asrp, &index);
    }
    
    msg = aap_msg_new();
    msg->create(msg, AAP_MSG_ASRP, AAP_MSG_IP,
		a->rseq, a->mseq, asrp, len);
    msg->send(msg, a->n, a->ns);
    a->mseq++;
                         
    return 0;
}

int aap_asrp_receive(aap *aap, aap_msg *msg){
    if(aap->cfg->scope == CFG_LARGE_SCOPE){
        /* We just reset the timer for the timeout. */
        aap->tim->unset(aap->tim, aap->asrp_timer);
        aap->asrp_timer = aap->cfg->ASRP_INTERVAL +
            (((double)rand()/RAND_MAX) * 0.6 - 0.3) * aap->cfg->ASRP_INTERVAL;
    }
    return 0;
}

static void delete(aap_asrp *a){
    int i = 0;
    for(i = 0;i < a->reports->length(a->reports);i++){
        asrp_report *report = a->reports->elem_at(a->reports, i);
        free(report);
    }
    a->reports->delete(a->reports);
    for(i = 0;i < a->requests->length(a->requests);i++){
        asrp_request *request = a->requests->elem_at(a->requests, i);
        free(request);
    }
    a->requests->delete(a->requests);
    free(a);
}

aap_asrp *aap_asrp_new(net *n, net_socket *ns, tim *t, cfg *c, aap *aap,
                       int rseq){
    plist *asa_ranges;
    list *ana_requests;
    int i = 0;
    aap_asrp *asrp = malloc(sizeof(aap_asrp));
    
    asrp->n = n;
    asrp->ns = ns;
    asrp->t = t;
    asrp->c = c;
    asrp->aap = aap;
    asrp->rseq = rseq;
    asrp->mseq = 0;
    
    asrp->send = send_msg;
    asrp->delete = delete;

    /* Create the report and request lists. */
    asrp->reports = list_new();
    asa_ranges = aap->arec->asa_ranges;
    for(i = 0;i < asa_ranges->length(asa_ranges);i++){
        addr_range *tmp_ar = asa_ranges->elem_at(asa_ranges, i);
        asrp_report *report = (asrp_report *)malloc(sizeof(asrp_report));
        report->first = tmp_ar->first_addr(tmp_ar);
        report->last = tmp_ar->last_addr(tmp_ar);
        report->in_use_count = aap->arec->in_use(aap->arec, tmp_ar);
        asrp->reports->insert(asrp->reports, report);
    }
    asrp->requests = list_new();
    ana_requests = aap->arec->ana_reqs;
    for(i = 0;i < ana_requests->length(ana_requests);i++){
        ana_req *tmp_ar = ana_requests->elem_at(ana_requests, i);
        asrp_request *request = (asrp_request *)malloc(sizeof(asrp_request));
        request->addrs_requested = tmp_ar->numaddr;
        request->end_time_requested = tmp_ar->end_time + tmp_ar->clock_skew;
        asrp->requests->insert(asrp->requests, request);
    }
    return asrp;
}

static char *add_raw8(char data, char *raw_message, int *index){
    raw_message = (char *)realloc(raw_message,
                                   (*index + 1) * sizeof(char));
    *(char *)(raw_message + *index) = data;
    *index = *index + 1;
    return raw_message;
}

static char *add_raw32(long data, char *raw_message, int *index){
    raw_message = (char *)realloc(raw_message,
                                   (*index + 4) * sizeof(char));
    *(long *)(raw_message + *index) = htonl(data);
    *index = *index + 4;
    return raw_message;
}
