%{
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "cfg.h"
#include "list.h"
#include "addr_range.h"
#include "allocation.h"
#include "ana_req.h"

extern char yytext[];
extern int column; 

list *cfgparser_list; 

static char *strtoname(char *str); 
int yyerror(char *s);
  
%}

%union {
    unsigned int num;
    unsigned long addr;
    char *str;
    list *l;
    addr_range *ar;
    ana_req *ana_r;
    allocation *alloc;
    cfg *cfg;
    struct constants {
        int STARTUP_WAIT, ANNOUNCE_WAIT, RESEND_WAIT, REPEAT_INTERVAL,
            ASA_INTERVAL, ASRP_INTERVAL;
    } constants;
    struct scope {
        unsigned long first, last;
        char large;
    } scope;
}

%token AAP_STARTUP_WAIT AAP_ANNOUNCE_WAIT AAP_RESEND_WAIT AAP_REPEAT_INTERVAL
%token AAP_ASA_INTERVAL AAP_ASRP_INTERVAL

%token ADDRESS PORT CACHE SCOPE LARGE

%token LBRACE RBRACE SEMICOLON
%token LPAREN RPAREN COMMA
%token ASSIGN

%token <num> CONSTANT
%token <str> STRING
%token <addr> IPADDR

%token MAAS PC ALLOCATION_RECORD
%token ASA ALLOCATION OWN ANA

%type <ar> asa
%type <alloc> alloc own
%type <ana_r> ana

%type <l> asa_list
%type <l> alloc_list own_list
%type <l> ana_list

%type <num> startup_wait announce_wait resend_wait repeat_interval
%type <num> asa_interval asrp_interval

%type <constants> aap_constants
%type <str> address cache
%type <num> port

%type <cfg> aap_cfg config_def

%type <l> config_list arec allocation_def

%type <scope> scope

%%

config
      : allocation_def { cfgparser_list = $1; }
      | config_list { cfgparser_list = $1; }
      ;

config_list
      : config_def { $$ = list_new(); $$->insert($$, $1); }
      | config_list config_def { $$ = $1; $$->insert($$, $2); }
      ;

allocation_def
      : ALLOCATION_RECORD STRING LBRACE arec RBRACE SEMICOLON { $$ = $4; }
      ;

config_def
      : PC STRING LBRACE aap_cfg RBRACE SEMICOLON
         { $$ = $4;
	   $$->name = strtoname($2);
	   $$->type = CFG_PC; }
      | MAAS STRING LBRACE aap_cfg RBRACE SEMICOLON
         { $$ = $4;
	   $$->name = strtoname($2);
	   $$->type = CFG_MAAS; }
      ;

aap_cfg
      : scope address port cache aap_constants 
        {
	  $$ = cfg_new();
	  $$->STARTUP_WAIT = $5.STARTUP_WAIT;
	  $$->ANNOUNCE_WAIT = $5.ANNOUNCE_WAIT;
	  $$->RESEND_WAIT = $5.RESEND_WAIT;
	  $$->REPEAT_INTERVAL = $5.REPEAT_INTERVAL;
	  $$->ASA_INTERVAL = $5.ASA_INTERVAL;
	  $$->ASRP_INTERVAL = $5.ASRP_INTERVAL;
	  $$->aap_addr = strtoname($2);
	  $$->aap_port = $3;
	  $$->arec.cache_fname = strtoname($4);
	  if($1.large == 0) {
	    $$->scope = addr_range_new($1.first, $1.last, (unsigned int)0xffffffff);
	  } else {
	    $$->scope = CFG_LARGE_SCOPE;
	  }
	}
      ;

aap_constants
      : startup_wait announce_wait resend_wait repeat_interval asa_interval asrp_interval
         { $$.STARTUP_WAIT = $1;
	   $$.ANNOUNCE_WAIT = $2;
	   $$.RESEND_WAIT = $3;
	   $$.REPEAT_INTERVAL = $4;
	   $$.ASA_INTERVAL = $5;
	   $$.ASRP_INTERVAL = $6; }
      ;

address
      : ADDRESS ASSIGN STRING SEMICOLON { $$ = $3; }
      ;

port
      : PORT ASSIGN CONSTANT SEMICOLON { $$ = $3; }
      ;

cache
      : CACHE ASSIGN STRING SEMICOLON { $$ = $3; }
      ;

scope
      : SCOPE ASSIGN LPAREN IPADDR COMMA IPADDR RPAREN SEMICOLON
         { $$.first = $4; $$.last = $6; $$.large = 0; }
      | SCOPE ASSIGN LARGE SEMICOLON
         { $$.large = 1; }
      ;

startup_wait
      : AAP_STARTUP_WAIT ASSIGN CONSTANT SEMICOLON { $$ = $3; }
      ;

announce_wait
      : AAP_ANNOUNCE_WAIT ASSIGN CONSTANT SEMICOLON { $$ = $3; }
      ;

resend_wait
      : AAP_RESEND_WAIT ASSIGN CONSTANT SEMICOLON { $$ = $3; }
      ;

repeat_interval
      : AAP_REPEAT_INTERVAL ASSIGN CONSTANT SEMICOLON { $$ = $3; }
      ;

asa_interval
      : AAP_ASA_INTERVAL ASSIGN CONSTANT SEMICOLON { $$ = $3; }
      ;

asrp_interval
      : AAP_ASRP_INTERVAL ASSIGN CONSTANT SEMICOLON { $$ = $3; }
      ;

arec 
      : asa_list alloc_list own_list ana_list
         { $$ = list_new(); $$->insert($$, $1); $$->insert($$, $2); $$->insert($$, $3); }
      ;


asa_list
      : asa { $$ = list_new(); $$->insert($$, $1); }
      | asa asa_list { $$ = $2; $$->insert($$, $1); }
      ;

alloc_list
      : { $$ = list_new(); } 
      | alloc { $$ = list_new(); $$->insert($$, $1); printf("list\n");}
      | alloc alloc_list { $$ = $2; $$->insert($$, $1); }
      ;

own_list
      : { $$ = list_new(); } 
      | own { $$ = list_new(); $$->insert($$, $1); }
      | own own_list { $$ = $2; $$->insert($$, $1); }
      ;

ana_list
      : { $$ = list_new(); }
      | ana { $$ = list_new(); $$->insert($$, $1); }
      | own own_list { $$ = $2; $$->insert($$, $1); }
      ;

asa
      : ASA LPAREN IPADDR COMMA IPADDR COMMA CONSTANT RPAREN SEMICOLON
         { $$ = addr_range_new($3, $5, $7); }
      ;

alloc
      : ALLOCATION LPAREN IPADDR COMMA IPADDR COMMA CONSTANT COMMA CONSTANT COMMA CONSTANT COMMA IPADDR RPAREN SEMICOLON
         { $$ = allocation_new($11, addr_range_new($3, $5, $7), $9, $13); }
      ;

own
      : OWN LPAREN IPADDR COMMA IPADDR COMMA CONSTANT COMMA CONSTANT COMMA CONSTANT RPAREN SEMICOLON
         { $$ = allocation_new($11, addr_range_new($3, $5, $7), $9, ALLOCATION_OWN); }
      ;

ana
      : ANA LPAREN CONSTANT COMMA CONSTANT COMMA CONSTANT COMMA CONSTANT RPAREN SEMICOLON
          { $$ = ana_req_create($3, $5, $7, $9); }
      ;

%%
int yyerror(char *s) {
        fflush(stdout);
        printf("\n%*s\n%*s\n", column, "^", column, s);
	return 1;
}

static char *strtoname(char *str) {
  char *s = strdup(str + 1);
  s[strlen(s) - 1] = 0;
  free(str);
  return s;
}
