#ifndef __LIST_H__
#define __LIST_H__

/**
 * list.h
 *
 * This module implements a doubly linked list which can
 * contain items of arbitrary types (void *).
 *
 * Implemented methods are
 *
 * void insert(list *l, void *item);
 *
 *   Inserts item into list l.
 *
 * int remove(list *l, int pos);
 *
 *   Removes the item at position pos from list l.
 *
 * int length(list *l);
 *
 *   Returns the length of the list l.
 *
 * int find(list *l, void *item);
 *
 *   Returns the position of item in list l.
 *
 * void *elem_at(list *l, int pos);
 *
 *   Returns the item at position pos.
 *
 * int is_empty(list *l);
 *
 *   Returns 1 is list is emptry, 0 otherwise.
 */

typedef struct list_ list;


struct list_node {
  void *item;
  struct list_node *next, *prev;
};



struct list_ {
  
  struct list_node *head, *last; /* first and last object in list */
  int len;                       /* length of list */
  
  void (* insert)(list *l, void *item);
  int (* remove)(list *l, int pos);
  
  int (* length)(list *l);
  int (* find)(list *l, void *item);
  void *(* elem_at)(list *l, int pos);
  
  int (* is_empty)(list *l);

  void (* delete)(list *l);
};


/* list_new returns a new list object */
list *list_new();

#endif
