
#include <stdlib.h>
#include "madcap_list.h"

/**
 * madcap_list.c
 *
 * Implements a doubly linked list. See list.h for details.
 */
 
  
static void insert(madcap_list *l, void *item);
static int remove(madcap_list *l, int pos);  
static int length(madcap_list *l);
static void *elem_at(madcap_list *l, int pos);
static int find(madcap_list *l, void *item);
static int is_empty(madcap_list *l);
static void delete(madcap_list *l);

/*------------------------------------------------------------------*/
static void insert(madcap_list *l, void *item) {
  struct madcap_list_node *node;

  node = malloc(sizeof(struct madcap_list_node));
  node->item = item;
  node->next = node->prev = NULL;
  
  l->len++;
  
  if(l->head == NULL) {
    l->head = l->last = node;
  } else {
    l->last->next = node;
    node->prev = l->last;
    l->last = node;
  }
}

/*------------------------------------------------------------------*/
static int remove(madcap_list *l, int pos) {
  struct madcap_list_node *node;
  int i;

  if(pos >= length(l)) {
    return -1;
  }
  
  node = l->head;
  for(i = 0; i < pos; i++) {
    node = node->next;
  }

  if(node->prev != NULL) {
    node->prev->next = node->next;
  }
  if(node->next != NULL) {
    node->next->prev = node->prev;
  }
  if(node == l->last) {
    l->last = node->prev;
  }
  if(node == l->head) {
    l->head = node->next;
  }
  l->len--;
  free(node);
  return 0;
}

/*------------------------------------------------------------------*/
static int length(madcap_list *l) {
  return l->len;
}

/*------------------------------------------------------------------*/
static int find(madcap_list *l, void *item) {
  struct madcap_list_node *node;
  int i;

  i = 0;
  node = l->head;
  while(node != NULL) {
    if(node->item == item) {
      return i;
    }
    node = node->next;
    i++;
  }
  return -1;
  
}

/*------------------------------------------------------------------*/
static void *elem_at(madcap_list *l, int pos) {
  struct madcap_list_node *node;
  int i;

  if(pos >= length(l)) {
    return NULL;
  }

  node = l->head;
  for(i = 0; i < pos; i++) {
    node = node->next;
  }

  return node->item;
  
}

/*------------------------------------------------------------------*/
static int is_empty(madcap_list *l) {
  return l->len == 0;
}

/*------------------------------------------------------------------*/
static void delete(madcap_list *l) {
  struct madcap_list_node *p, *n = l->head;
  while(n != NULL) {
    p = n->next;
    free(n);
    n = p;
  }
  free(l);
}
/*------------------------------------------------------------------*/
madcap_list *madcap_list_new() {
  madcap_list *l;
  if((l = calloc(1, sizeof(madcap_list))) == NULL) {
    return NULL;
  }

  l->head = l->last = NULL;
  l->len      = 0;
  
  l->insert   = insert;
  l->remove   = remove;
  l->length   = length;
  l->find     = find;
  l->elem_at  = elem_at;
  l->is_empty = is_empty;
  l->delete   = delete;
  return l;
 
}
