#ifndef __TIM_H__
#define __TIM_H__

#include <sys/time.h>
#include "plist.h"

/**
 * tim.h
 *
 * Implements a timer with methods for inserting and extrating
 * time events. Each time event has an associated callback function,
 * which is called when the timer expires.
 *
 * The methods are
 *
 * tim_id_t set(tim *tim, int time, tim_callback callback, void *obj);
 *
 *   Sets a timer to the time specified by the time argument (seconds). When
 *   this timer expires, the function given by callback will be called, with
 *   obj as its first argument. Return value is an identifier for this time
 *   event.
 *
 * int unset(tim *tim, tim_id_t id);
 *
 *   Unsets (removes) the time event specified by id.
 *
 * int get_next_time(tim *tim, struct timeval *t);
 *
 *   Returns the time of to the next time event in the timeval struct pointed
 *   to by t.
 *
 * int call_next(tim *tim);
 *
 *   Calls the callback function for the next time event, and removes (unsets
 *   the event.
 *
 * int is_empty(tim *tim);
 *
 *   Returns 1 if there are no queued time events.
 *
 */


typedef struct tim_ tim;
typedef int tim_id_t;
typedef int (* tim_callback)(void *, unsigned int);

struct tim_ {
  int sequence_number;
  plist *events;
  tim_id_t (* set)(tim *tim, int time, tim_callback callback, void *obj);
  int (* unset)(tim *tim, tim_id_t id);
  int (* get_next_time)(tim *tim, struct timeval *t);
  int (* call_next)(tim *tim);
  /*  int (* unset_next)(tim *tim);*/
  int (* is_empty)(tim *tim);
};

tim *tim_new();

#endif /* __TIM_H__ */
